@@ -18,7 +18,6 @@ from i2.signatures import (
     KO,  # KEYWORD_ONLY
 )
 
-
 from lined.util import signature_from_first_and_last_func, func_name
 
 # MultiFuncSpec = Dict[str, Callable]
@@ -102,11 +101,40 @@ _line_init_reserved_names = {"pipeline_name", "input_name", "output_name"}
 
 
 def _func_to_name_func_pair(func):
-    """The func.__name__ of a callable func, or makes and returns one if that fails.
-    To make one, it calls unamed_func_name which produces incremental names to reduce the chances of clashing"""
+    """
+    From a function func, returns a pair (name_of_func, func) where name_of_func is either the existing string
+    func.__name__ or an inferred string using func_name. If func is instead already a pair, a check is performed
+    to ensure that it is of the expected form, i.e., in a (str, callable) form
+
+    :param func: callable or a pair, in the latter case a check is performed
+    :return: (str, callable) pair, where the string is the name of the function
+
+    >>> def my_func(x):
+    ...    return x + 1
+    >>> assert _func_to_name_func_pair(my_func)[0] == 'my_func'
+    >>> assert _func_to_name_func_pair(my_func)[1] is my_func
+
+    A function with no .__name__ attribute, such as a lambda, will be given a unique name automatically in the form of
+    f'unnamed_func_{i}'. The uniqueness is achieved by incrementing i.
+
+    >>> my_lambda_func = lambda x: 2 * x
+    >>> given_name, func = _func_to_name_func_pair(my_lambda_func)
+    >>> assert given_name.startswith('unnamed_func_')
+
+    >>> class my_class:
+    ...    def my_class_method(self, x):
+    ...        return x + 2 * x
+    >>> assert _func_to_name_func_pair(my_class.my_class_method)[0] == 'my_class_method'
+
+    """
     if isinstance(func, tuple) and len(func) == 2:
+        # We could just return func here, but to be clear...
+        # func is actually a name func pair, so let's extract the name and the func
         name, func = func
-        return name, func
+        # and assert these are in fact a name and a function
+        assert isinstance(name, str)
+        assert callable(func)
+        return name, func  # an finally returning it
     else:
         return func_name(func), func
 
@@ -118,8 +146,13 @@ def _merge_funcs_and_named_funcs(funcs, named_funcs):
         "Can't name a function with any of the following strings: "
         f"{', '.join(_line_init_reserved_names)}"
     )
+
     funcs_obtained_from_named_funcs = tuple(named_funcs.values())
-    named_funcs_obtained_from_funcs = dict(map(_func_to_name_func_pair, funcs))
+    named_funcs_obtained_from_funcs = map(_func_to_name_func_pair, funcs)
+    # make sure the names are unique, by adding a suffix to some of the repeating names if necessary
+    named_funcs_obtained_from_funcs = dict(
+        uniquize_funcs_names(named_funcs_obtained_from_funcs)
+    )
 
     assert named_funcs_obtained_from_funcs.keys().isdisjoint(
         named_funcs
@@ -128,12 +161,33 @@ def _merge_funcs_and_named_funcs(funcs, named_funcs):
         tuple(named_funcs_obtained_from_funcs.values())
         + funcs_obtained_from_named_funcs
     )
+
+    named_funcs = dict(named_funcs_obtained_from_funcs, **named_funcs)
+    # print(named_funcs.values())
+    # print(funcs)
+    # assert set(named_funcs.values()) == set(funcs), "Some of the functions have the same name," \
+    #                                             " please explicitly provide disjoint names"
+
     return (
         funcs,
-        dict(named_funcs_obtained_from_funcs, **named_funcs),
+        named_funcs,
     )
 
 
+def uniquize_funcs_names(named_funcs_obtained_from_funcs):
+    all_func_names = []
+    prefix_counter = 0
+    unique_named_funcs_obtained_from_funcs = []
+    for func_name_pair in named_funcs_obtained_from_funcs:
+        name, func = func_name_pair
+        if name in all_func_names:
+            name = name + f"_{prefix_counter}"
+            prefix_counter += 1
+        all_func_names.append(name)
+        unique_named_funcs_obtained_from_funcs.append((name, func))
+    return unique_named_funcs_obtained_from_funcs
+
+
 def _mk_first_argument_position_only(func):
     """Replaces the first argument's parameter kind by PO (POSITION_ONLY).
     This is needed in some edge cases that involved functions that work only with PO
@@ -387,8 +441,15 @@ class Line:
                 # TODO: Get rid of this if/when we get rid of forced position only
                 first_fnode, *_ = funcs
                 underlying_funcs_sig = Sig(first_fnode.func)
-                sig = Sig(underlying_funcs_sig)
-                funcs[0] = fnode(sig(first_fnode.func), first_fnode.name)
+                if not hasattr(first_fnode.func, "__func__"):
+                    sig = Sig(underlying_funcs_sig)
+                    funcs[0] = fnode(sig(first_fnode.func), first_fnode.name)
+                else:  # first_fnode.func is a method, and we get
+                    # AttributeError: 'method' object has no attribute '__signature__'
+                    # when trying in change signature
+                    # To reproduce, remove not hasattr(first_fnode.func, '__func__')
+                    # condition
+                    pass
             return type(self)(*funcs, pipeline_name=f"{pipeline_name}[{item_str}]")
         else:
             raise TypeError(f"Don't know how to handle that type of key: {k}")
@@ -792,7 +853,8 @@ class LineParametrized(Line):
 
         def sig_without_the_first_input(func):
             sig = Sig(func)
-            sig = sig - sig.names[0]
+            if len(sig.names) > 0:
+                sig = sig - sig.names[0]
             return sig.ch_kinds(**{name: KO for name in sig.names})
 
         # _funcs = map(normalize_func, _funcs)  # TODO: Test edge cases to assess need
@@ -1017,6 +1079,7 @@ mk_multi_func = ParallelFuncs  # back-compatibility alias
 
 from collections import defaultdict
 
+
 # Class to represent a graph
 class Digraph:
     """This class is experiemental and will probably move to meshed."""
@@ -1063,3 +1126,36 @@ class Digraph:
 
                 # Print contents of stack
         return stack
+
+
+if __name__ == "__main__":
+    from lined.base import _merge_funcs_and_named_funcs
+    import pytest
+
+    class my_class:
+        """A simple class with one method"""
+
+        def add_one(self, x):
+            return x + 1
+
+    class my_other_class:
+        """Another simple class with one method"""
+
+        def add_one(self, x):
+            return x + 2
+
+    first = my_class()
+    f = first.add_one
+
+    second = my_class()
+    g = second.add_one
+
+    third = my_other_class()
+    h = third.add_one
+
+    i = lambda x: 2 * x
+
+    def j(x: int):
+        return 2 * x + 1
+
+    print(len(_merge_funcs_and_named_funcs((f, g, h, i), named_funcs={})[0]))
