@@ -5,7 +5,7 @@ from functools import partial, wraps
 from collections import deque
 from typing import Any, Mapping
 from dataclasses import dataclass
-from operator import not_
+from operator import not_, methodcaller
 
 from lined.util import func_name, partial_plus, n_required_args
 from lined.simple import Pipe
@@ -165,7 +165,19 @@ class Command:
     >>> command()
     hello, world
 
+    Note that the same can be achieved with
+    `partial(func, *args, **kwargs)`.
+
+    >>> from operator import methodcaller
+    >>> from functools import partial
+    >>> def mk_command(func, *args, **kwargs):
+    ...     return partial(methodcaller('__call__', *args, **kwargs), func)
+    >>> command = mk_command(print, 'hello', 'world', sep=', ')
+    >>> command()
+    hello, world
+
     See: https://en.wikipedia.org/wiki/Command_pattern
+
     """
 
     def __init__(self, func, *args, **kwargs):
@@ -175,6 +187,65 @@ class Command:
         return self.func(*self.args, **self.kwargs)
 
 
+class CommandIter(Command):
+    """An infinite iterator that returns the results of a Command called repeatedly.
+
+    Might become deprecated:
+    Use `iter(partial(func, *args, **kwargs), object())` instead.
+
+    >>> from random import uniform
+    >>> from itertools import islice
+    >>> it = CommandIter(uniform, 0, 10)
+    >>> rand_nums = list(islice(it, 4))
+    >>> assert len(rand_nums) == 4
+    >>> rand_nums  # doctest: +SKIP
+    [4.48171445690221, 7.466083642212892, 0.24120342781796422, 3.694956861724484]
+    """
+
+    def __iter__(self):
+        while True:
+            yield self()
+
+
+def functioncaller(*args, **kwargs):
+    """Call a function given positional and keyword arguments.
+
+
+    >>> import operator
+    >>> import functools
+    >>> from lined import Pipe
+    >>> f = Pipe(
+    ...     functools.partial(getattr, operator),  # get an operator func by name
+    ...     functioncaller(49, 7)  # apply it to 49 and 7
+    ... )
+    >>> f('add')
+    56
+    >>> f('sub')
+    42
+
+    Note: functioncaller just returns
+     `operator.methodcaller('__call__', *args, **kwargs)`.
+
+    """
+    return methodcaller("__call__", *args, **kwargs)
+
+
+def call(func):
+    """Just call the input function with not arguments.
+
+    Equivalent to `functioncaller()`
+
+    >>> from lined import Line
+    >>> from functools import partial
+    >>>
+    >>> line = Line(lambda x: partial(print, f"{x*3=}"), call)
+    >>> line(14)
+    x*3=42
+
+    """
+    return func()
+
+
 class ItemsNotSorted(RuntimeError):
     """Use to indicate that two consecutive items where not in the expected
     order"""
